# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore

from prettyqt import core


QtCore.QTranslator.__bases__ = (core.Object,)


class Translator(QtCore.QTranslator):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QFrame.__bases__ = (widgets.Widget,)


class Frame(QtWidgets.QFrame):
    pass



# -*- coding: utf-8 -*-

"""custom_models module

contains custom models
"""


from .pathvalidator import PathValidator
from .notemptyvalidator import NotEmptyValidator
from .notzerovalidator import NotZeroValidator
from .compositevalidator import CompositeValidator

__all__ = ["PathValidator", "NotEmptyValidator", "NotZeroValidator", "CompositeValidator"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import datetime

from qtpy import QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict


SELECTION_MODES = bidict(none=QtWidgets.QCalendarWidget.NoSelection,
                         single=QtWidgets.QCalendarWidget.SingleSelection)

HEADER_FORMATS = bidict(none=QtWidgets.QCalendarWidget.NoVerticalHeader,
                        week_numbers=QtWidgets.QCalendarWidget.ISOWeekNumbers)


QtWidgets.QCalendarWidget.__bases__ = (widgets.Widget,)


class CalendarWidget(QtWidgets.QCalendarWidget):

    def __getstate__(self):
        return dict(date=self.selectedDate())

    def __setstate__(self, state):
        self.__init__()
        self.setSelectedDate(state["date"])

    def get_date(self) -> datetime.date:
        try:
            return self.selectedDate().toPyDate()  # pyqt5
        except (TypeError, AttributeError):
            return self.selectedDate().toPython()

    def get_value(self) -> datetime.date:
        return self.get_date()

    def set_value(self, value):
        self.setSelectedDate(value)


if __name__ == "__main__":
    app = widgets.app()
    w = CalendarWidget()
    w.show()
    app.exec_()



# -*- coding: utf-8 -*-

"""syntaxhighlighters module

contains some custom syntax highlighers
"""

from .jsonhighlighter import JsonHighlighter
from .pythonhighlighter import PythonHighlighter
from .yamlhighlighter import YamlHighlighter

__all__ = ["JsonHighlighter", "YamlHighlighter", "PythonHighlighter"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import datetime

from qtpy import QtWidgets

from prettyqt import core, widgets


QtWidgets.QDateTimeEdit.__bases__ = (widgets.AbstractSpinBox,)


class DateTimeEdit(QtWidgets.QDateTimeEdit):

    value_changed = core.Signal(datetime.datetime)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setCalendarPopup(True)
        self.dateTimeChanged.connect(self.datetime_changed)

    def datetime_changed(self, date):
        dt = self.get_datetime()
        self.value_changed.emit(dt)

    def __getstate__(self):
        return dict(calendar_popup=self.calendarPopup(),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip(),
                    datetime=self.get_datetime(),
                    range=(self.min_datetime(), self.max_datetime()),
                    display_format=self.displayFormat(),
                    enabled=self.isEnabled())

    def __setstate__(self, state):
        self.__init__(state["datetime"])
        self.setDateTime(state["datetime"])
        self.setEnabled(state.get("enabled", True))
        self.set_range(*state["range"])
        self.setDisplayFormat(state["display_format"])
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def set_range(self, lower: datetime.datetime, upper: datetime.datetime):
        self.setToolTip(f"{lower} <= x <= {upper}")
        self.setDateTimeRange(lower, upper)

    def set_format(self, fmt: str):
        self.setDisplayFormat(fmt)

    def get_value(self) -> datetime.datetime:
        return self.get_datetime()

    def set_value(self, value: datetime.datetime):
        self.setDateTime(value)

    def get_datetime(self) -> datetime.datetime:
        try:
            return self.dateTime().toPython()
        except TypeError:
            return self.dateTime().toPyDateTime()

    def min_datetime(self) -> datetime.datetime:
        try:
            return self.minimumDateTime().toPython()
        except (TypeError, AttributeError):
            return self.minimumDateTime().toPyDateTime()

    def max_datetime(self) -> datetime.datetime:
        try:
            return self.maximumDateTime().toPython()
        except (TypeError, AttributeError):
            return self.maximumDateTime().toPyDateTime()

    def min_date(self) -> datetime.date:
        try:
            return self.minimumDate().toPython()
        except (TypeError, AttributeError):
            return self.minimumDate().toPyDate()

    def max_date(self) -> datetime.date:
        try:
            return self.maximumDate().toPython()
        except (TypeError, AttributeError):
            return self.maximumDate().toPyDate()

    def get_date(self) -> datetime.date:
        try:
            return self.date().toPython()
        except (TypeError, AttributeError):
            return self.date().toPyDate()

    def min_time(self) -> datetime.time:
        try:
            return self.minimumTime().toPython()
        except (TypeError, AttributeError):
            return self.minimumTime().toPyTime()

    def max_time(self) -> datetime.time:
        try:
            return self.maximumTime().toPython()
        except (TypeError, AttributeError):
            return self.maximumTime().toPyTime()

    def get_time(self) -> datetime.time:
        try:
            return self.time().toPython()
        except (TypeError, AttributeError):
            return self.time().toPyTime()


if __name__ == "__main__":
    app = widgets.app()
    widget = DateTimeEdit()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets

from prettyqt import core, widgets
from prettyqt.utils import bidict


TICK_POSITIONS = bidict(none=QtWidgets.QSlider.NoTicks,
                        both_sides=QtWidgets.QSlider.TicksBothSides,
                        above=QtWidgets.QSlider.TicksAbove,
                        below=QtWidgets.QSlider.TicksBelow)

ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal,
                      vertical=QtCore.Qt.Vertical)


QtWidgets.QAbstractSlider.__bases__ = (widgets.Widget,)


class AbstractSlider(QtWidgets.QAbstractSlider):

    value_changed = core.Signal(int)

    def __init__(self, orientation="horizontal", parent=None):
        super().__init__(ORIENTATIONS[orientation], parent)
        self.valueChanged.connect(self.on_value_change)

    def on_value_change(self):
        self.value_changed.emit(self.value())

    def __getstate__(self):
        return dict(range=(self.minimum(), self.maximum()),
                    value=self.value(),
                    has_tracking=self.hasTracking(),
                    inverted_controls=self.invertedControls(),
                    inverted_appearance=self.invertedAppearance(),
                    single_step=self.singleStep(),
                    page_step=self.pageStep())

    def __setstate__(self, state):
        self.__init__()
        self.set_range(*state["range"])
        self.set_value(state["value"])
        self.setSingleStep(state["single_step"])
        self.setPageStep(state["page_step"])
        self.setTracking(state["has_tracking"])
        self.setInvertedControls(state["inverted_controls"])
        self.setInvertedAppearance(state["inverted_appearance"])

    def is_horizontal(self) -> bool:
        """check if silder is horizontal

        Returns:
            True if horizontal, else False
        """
        return self.orientation() == QtCore.Qt.Horizontal

    def is_vertical(self) -> bool:
        """check if silder is vertical

        Returns:
            True if vertical, else False
        """
        return self.orientation() == QtCore.Qt.Vertical

    def set_horizontal(self):
        """set slider orientation to horizontal
        """
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        """set slider orientation to vertical
        """
        self.setOrientation(QtCore.Qt.Vertical)

    def scroll_to_min(self):
        """scroll to the minimum value of the slider
        """
        self.setValue(self.minimum())

    def scroll_to_max(self):
        """scroll to the maximum value of the slider
        """
        self.setValue(self.maximum())

    def set_range(self, min_val, max_val):
        self.setRange(min_val, max_val)

    def get_value(self):
        return super().value()

    def set_value(self, value: int):
        self.setValue(value)


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    slider = AbstractSlider()
    slider.setRange(0, 100)
    slider.value_changed.connect(print)
    slider.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets

from prettyqt import widgets
from prettyqt.utils import bidict


ORIENTATIONS = bidict(horizontal=QtCore.Qt.Horizontal,
                      vertical=QtCore.Qt.Vertical)


QtWidgets.QSplitter.__bases__ = (widgets.Frame,)


class Splitter(QtWidgets.QSplitter):

    def __init__(self, orientation="horizontal", parent=None):
        super().__init__(ORIENTATIONS[orientation], parent)

    def __getitem__(self, index):
        if isinstance(index, int):
            return self.widget(index)
        else:
            return self.findChild(QtWidgets.QWidget, index)

    def __getstate__(self):
        return dict(items=self.get_children(),
                    orientation=self.get_orientation(),
                    handle_width=self.handleWidth(),
                    children_collapsible=self.childrenCollapsible(),
                    opaque_resize=self.opaqueResize())

    def __setstate__(self, state):
        self.__init__(state["orientation"])
        for item in state["items"]:
            self.addWidget(item)
        self.setHandleWidth(state["handle_width"])
        self.setChildrenCollapsible(state["children_collapsible"])
        self.setOpaqueResize(state["opaque_resize"])

    def __iter__(self):
        return iter(self.get_children())

    def __len__(self):
        return self.count()

    def __add__(self, other):
        if isinstance(other, (QtWidgets.QLayout, QtWidgets.QWidget)):
            self.add(other)
            return self
        raise TypeError(f"Invalid type: {type(other)}")

    def get_children(self) -> list:
        return [self[i] for i in range(self.count())]

    def add_widget(self, widget: QtWidgets.QWidget):
        self.addWidget(widget)

    def add_layout(self, layout: QtWidgets.QLayout):
        widget = widgets.Widget()
        widget.set_layout(layout)
        self.addWidget(widget)

    def add(self, item):
        if isinstance(item, QtWidgets.QWidget):
            self.add_widget(item)
        else:
            self.add_layout(item)

    @classmethod
    def from_widgets(cls, widgets, horizontal: bool = False, parent=None):
        orientation = "horizontal" if horizontal else "vertical"
        splitter = cls(orientation, parent=parent)
        for widget in widgets:
            splitter += widget
        return splitter

    def set_orientation(self, orientation: str):
        """set the orientation of the splitter

        Allowed values are "horizontal", "vertical"

        Args:
            mode: orientation for the splitter

        Raises:
            ValueError: orientation does not exist
        """
        if orientation not in ORIENTATIONS:
            raise ValueError(f"{orientation} not a valid orientation.")
        self.setOrientation(ORIENTATIONS[orientation])

    def get_orientation(self) -> str:
        """returns current orientation

        Possible values: "horizontal", "vertical"

        Returns:
            orientation
        """
        return ORIENTATIONS.inv[self.orientation()]


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    widget = Splitter()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QWidgetAction.__bases__ = (widgets.Action,)


class WidgetAction(QtWidgets.QWidgetAction):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import pathlib

from prettyqt import gui


class PathValidator(gui.Validator):

    def __getstate__(self):
        return dict()

    def __setstate__(self, state):
        self.__init__()

    def validate(self, text: str, pos: int = 0):
        if pathlib.Path(text).exists():
            return (self.Acceptable, text, pos)
        return (self.Intermediate, text, pos)


if __name__ == "__main__":
    from prettyqt import widgets
    val = PathValidator()
    app = widgets.app()
    widget = widgets.LineEdit("This is a test")
    widget.setValidator(val)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

from prettyqt import widgets
from prettyqt.utils import bidict


POPUP_MODES = bidict(delayed=QtWidgets.QToolButton.DelayedPopup,
                     menu_button=QtWidgets.QToolButton.MenuButtonPopup,
                     instant=QtWidgets.QToolButton.InstantPopup)

ARROW_TYPES = bidict(none=QtCore.Qt.NoArrow,
                     up=QtCore.Qt.UpArrow,
                     down=QtCore.Qt.DownArrow,
                     left=QtCore.Qt.LeftArrow,
                     right=QtCore.Qt.RightArrow)


QtWidgets.QToolButton.__bases__ = (widgets.AbstractButton,)


class ToolButton(QtWidgets.QToolButton):

    @classmethod
    def for_menu(cls, menu, icon=None):
        btn = cls()
        btn.setMenu(menu)
        btn.set_popup_mode("instant")
        btn.set_icon(icon)
        return btn

    def set_popup_mode(self, mode: str):
        """sets the popup mode of the toolbutton

        valid values are: "delayed", "menu_button", "instant"

        Args:
            mode: popup mode to use

        Raises:
            ValueError: invalid popup mode
        """
        if mode not in POPUP_MODES:
            raise ValueError("Invalid mode.")
        self.setPopupMode(POPUP_MODES[mode])

    def get_popup_mode(self) -> str:
        """returns popup mode

        possible values are "delayed", "menu_button", "instant"

        Returns:
            popup mode
        """
        return POPUP_MODES.inv[self.popupMode()]

    def set_arrow_type(self, mode: str):
        """sets the arrow type of the toolbutton

        valid values are: "delayed", "menu_button", "instant"

        Args:
            mode: arrow type to use

        Raises:
            ValueError: invalid arrow type
        """
        if mode not in ARROW_TYPES:
            raise ValueError("Invalid arrow type.")
        self.setArrowType(ARROW_TYPES[mode])

    def get_arrow_type(self) -> str:
        """returns arrow type

        possible values are "delayed", "menu_button", "instant"

        Returns:
            arrow type
        """
        return ARROW_TYPES.inv[self.arrowType()]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import gui, widgets, core
from prettyqt.utils import bidict


ECHO_MODES = bidict(normal=QtWidgets.QLineEdit.Normal,
                    no_echo=QtWidgets.QLineEdit.NoEcho,
                    password=QtWidgets.QLineEdit.Password,
                    echo_on_edit=QtWidgets.QLineEdit.PasswordEchoOnEdit)


QtWidgets.QLineEdit.__bases__ = (widgets.Widget,)


class LineEdit(QtWidgets.QLineEdit):

    value_changed = core.Signal(str)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.textChanged.connect(self.set_validation_color)
        self.textChanged.connect(self.value_changed)

    def __repr__(self):
        return f"LineEdit: {self.__getstate__()}"

    def __getstate__(self):
        return dict(text=self.text(),
                    enabled=self.isEnabled(),
                    tooltip=self.toolTip(),
                    statustip=self.statusTip(),
                    font=gui.Font(self.font()),
                    validator=self.validator(),
                    max_length=self.maxLength(),
                    read_only=self.isReadOnly(),
                    input_mask=self.inputMask(),
                    has_frame=self.hasFrame(),
                    placeholder_text=self.placeholderText())

    def __setstate__(self, state):
        self.__init__()
        self.set_text(state["text"])
        self.setEnabled(state.get("enabled", True))
        self.setFont(state["font"])
        self.setValidator(state["validator"])
        self.setInputMask(state["input_mask"])
        self.setMaxLength(state["max_length"])
        self.setPlaceholderText(state["placeholder_text"])
        self.setReadOnly(state["read_only"])
        self.setFrame(state["has_frame"])
        self.setToolTip(state.get("tooltip", ""))
        self.setStatusTip(state.get("statustip", ""))

    def __add__(self, other):
        if isinstance(other, str):
            self.append_text(other)
            return self

    def font(self) -> gui.Font:
        return gui.Font(super().font())

    def set_font(self,
                 font_name: str,
                 font_size: int = -1,
                 weight: int = -1,
                 italic: bool = False):
        font = gui.Font(font_name, font_size, weight, italic)
        self.setFont(font)

    def append_text(self, text: str):
        self.set_text(self.text() + text)

    def set_text(self, text: str):
        self.setText(text)

    def set_read_only(self, value: bool = True):
        """set test to read only

        Args:
            value: True, for read-only, otherwise False
        """
        self.setReadOnly(value)

    def set_regex_validator(self, regex: str) -> gui.RegExpValidator:
        validator = gui.RegExpValidator(self)
        validator.set_regex(regex)
        self.setValidator(validator)
        return validator

    def set_range(self, lower, upper):
        val = gui.IntValidator()
        val.setRange(lower, upper)
        self.set_validator(val)

    def set_validator(self, validator: gui.Validator):
        self.setValidator(validator)

    def set_input_mask(self, mask: str):
        self.setInputMask(mask)

    def set_color(self, color: str):
        self.setStyleSheet(f"background-color: {color};")

    def set_validation_color(self, state: bool):
        color = "rgb(255, 175, 90)" if not self.is_valid() else "white"
        self.set_color(color)

    def set_echo_mode(self, mode: str):
        """set echo mode

        Valid values are "normal", "no_echo", "password", "echo_on_edit"

        Args:
            policy: echo mode to use

        Raises:
            ValueError: invalid echo mode
        """
        if mode not in ECHO_MODES:
            raise ValueError("Invalid echo mode")
        self.setEchoMode(ECHO_MODES[mode])

    def get_echo_mode(self) -> str:
        """returns echo mode

        possible values are "normal", "no_echo", "password", "echo_on_edit"

        Returns:
            echo mode
        """
        return ECHO_MODES.inv[self.echoMode()]

    def set_value(self, value: str):
        self.setText(value)

    def get_value(self) -> str:
        return self.text()

    def is_valid(self) -> bool:
        return self.hasAcceptableInput()


if __name__ == "__main__":
    app = widgets.app()
    widget = LineEdit("This is a test")
    widget.set_regex_validator("[0-9]+")
    widget.setFont(gui.Font("Consolas"))
    widget.show()
    app.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `prettyqt` package."""

import pickle
import pathlib

from prettyqt import custom_validators, widgets

test_widget = widgets.Widget()


def test_pathvalidator():
    val = custom_validators.PathValidator()
    with open("data.pkl", "wb") as jar:
        pickle.dump(val, jar)
    with open("data.pkl", "rb") as jar:
        val = pickle.load(jar)
    assert val.is_valid_value(str(pathlib.Path.cwd()))
    assert not val.is_valid_value("abs")
    repr(val)


def test_notzerovalidator():
    val = custom_validators.NotZeroValidator()
    with open("data.pkl", "wb") as jar:
        pickle.dump(val, jar)
    with open("data.pkl", "rb") as jar:
        val = pickle.load(jar)
    assert val.is_valid_value("1")
    assert not val.is_valid_value("0")
    repr(val)


def test_notemptyvalidator():
    val = custom_validators.NotEmptyValidator()
    with open("data.pkl", "wb") as jar:
        pickle.dump(val, jar)
    with open("data.pkl", "rb") as jar:
        val = pickle.load(jar)
    assert val.is_valid_value("1")
    assert not val.is_valid_value("")
    repr(val)


def test_compositevalidator():
    val1 = custom_validators.NotEmptyValidator()
    val2 = custom_validators.NotZeroValidator()
    val = custom_validators.CompositeValidator([val1, val2])
    with open("data.pkl", "wb") as jar:
        pickle.dump(val, jar)
    with open("data.pkl", "rb") as jar:
        val = pickle.load(jar)
    assert val.is_valid_value("1")
    assert not val.is_valid_value("")
    assert not val.is_valid_value("0")
    val1 + val2
    repr(val)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import custom_widgets, widgets


class DataItem(object):
    def __init__(self, label, *args, **kwargs):
        self.label = label
        self.colspan = 1
        self.label_col = 0
        self.active_on = list()
        self.not_active_on = list()

    def __get__(self, instance, owner):
        return self.widget.get_value()

    def set_pos(self, col=0, colspan=None):
        """
        Set data item's position on a GUI layout
        """
        self.label_col = col
        self.colspan = colspan
        return self

    def store(self, prop):
        # self.set_prop("display", store=prop)
        return self

    def set_active(self, prop):
        # self.set_prop("display", active=prop)
        self.active_on.append(prop)
        return self

    def set_not_active(self, prop):
        # self.set_prop("display", active=prop)
        self.not_active_on.append(prop)
        return self


class FloatItem(DataItem):
    """
    Construct a float data item
        * label [string]: name
        * default [float]: default value (optional)
        * min [float]: minimum value (optional)
        * max [float]: maximum value (optional)
        * slider [bool]: if True, shows a slider widget right after the line
          edit widget (default is False)
        * step [float]: step between tick values with a slider widget (optional)
        * unit [string]: physical unit (optional)
        * check [bool]: if False, value is not checked (optional, default=True)
    """

    def __init__(self, label, default=None, min_val=None, max_val=None,
                 unit="", step=0.1, slider=False, check=True):
        super().__init__(label, default=default, min_val=min_val,
                         max_val=max_val, unit=unit, check=check)
        self.widget = widgets.DoubleSpinBox()
        self.widget.set_range(min_val, max_val)
        self.widget.setSingleStep(step)
        self.widget.setSuffix(unit)
        if default is not None:
            self.widget.set_value(default)


class IntItem(DataItem):
    """
    Construct an integer data item
        * label [string]: name
        * default [int]: default value (optional)
        * min [int]: minimum value (optional)
        * max [int]: maximum value (optional)
        * unit [string]: physical unit (optional)
        * slider [bool]: if True, shows a slider widget right after the line
          edit widget (default is False)
        * check [bool]: if False, value is not checked (optional, default=True)
    """

    def __init__(self, label, default=None, min_val=0, max_val=None,
                 unit="", step=1, slider=False, check=True):
        super().__init__(label, default=default, min_val=min_val, max_val=max_val,
                         unit=unit, check=check)
        if slider:
            self.widget = widgets.Slider()
        else:
            self.widget = widgets.SpinBox()
            self.widget.setSuffix(unit)
        self.widget.set_range(min_val, max_val)
        self.widget.setSingleStep(step)
        if default is not None:
            self.widget.set_value(default)


class StringItem(DataItem):
    """
    Construct a string data item
        * label [string]: name
        * default [string]: default value (optional)
        * notempty [bool]: if True, empty string is not a valid value (opt.)
    """

    def __init__(self, label, default=None, notempty=None,
                 regex=None):
        super().__init__(label, default=default)
        self.widget = widgets.LineEdit()
        if default is not None:
            self.widget.set_value(default)


class BoolItem(DataItem):
    """
    Construct a boolean data item
        * text [string]: form's field name (optional)
        * label [string]: name
        * default [string]: default value (optional)
        * check [bool]: if False, value is not checked (optional, default=True)
    """

    def __init__(self, label, default=None, check=True, use_push=False):
        super().__init__(label, default=default, check=check)
        if use_push:
            self.widget = widgets.PushButton()
            self.widget.setCheckable(True)
        else:
            self.widget = widgets.CheckBox()
        if default is not None:
            self.widget.set_value(default)


class ColorItem(DataItem):
    """
    Construct a color data item
        * label [string]: name
        * default [string]: default value (optional)
        * check [bool]: if False, value is not checked (optional, default=True)

    Color values are encoded as hexadecimal strings or Qt color names
    """

    def __init__(self, label, default=None, check=True):
        super().__init__(label, default=default, check=check)
        self.widget = custom_widgets.ColorChooserButton()
        if default is not None:
            self.widget.set_value(default)


class ChoiceItem(DataItem):
    """
    Construct a data item for a list of choices.
        * label [string]: name
        * choices [list, tuple or function]: string list or (key, label) list
          function of two arguments (item, value) returning a list of tuples
          (key, label, image) where image is an icon path, a QIcon instance
          or a function of one argument (key) returning a QIcon instance
        * default [-]: default label or default key (optional)
        * check [bool]: if False, value is not checked (optional, default=True)
        * radio [bool]: if True, shows radio buttons instead of a combo box
          (default is False)
    """

    def __init__(self, label, choices, default=None, check=True, radio=False):
        super().__init__(label, default=default, check=check)
        if radio:
            self.widget = custom_widgets.SelectionWidget(layout="vertical")
        else:
            self.widget = widgets.ComboBox()
        for item in choices:
            if isinstance(item, tuple):
                if len(item) == 2:
                    self.widget.add(item[1], item[0])
                elif len(item) == 3:
                    self.widget.add(item[1], item[0], item[2])
            else:
                self.widget.add(item)
        if default is not None:
            self.widget.set_value(default)


class ImageChoiceItem(ChoiceItem):
    pass


class MultipleChoiceItem(DataItem):

    def __init__(self, label, choices, default=None, check=True):
        super().__init__(label, default=default, check=check)
        self.widget = widgets.ListWidget()
        self.widget.set_selection_mode("multi")
        for item in choices:
            if isinstance(item, tuple):
                if len(item) == 2:
                    self.widget.add(item[1], item[0])
                elif len(item) == 3:
                    self.widget.add(item[1], item[0], item[2])
            else:
                self.widget.add(item)
        if default is not None:
            self.widget.set_value(default)


class FileSaveItem(DataItem):

    def __init__(self, label, formats="*", default=None, basedir=None, check=True):
        super().__init__(label, default=default, check=check)
        self.widget = custom_widgets.FileChooserButton()
        if default is not None:
            self.widget.set_value(default)


class ButtonItem(DataItem):
    """
    Construct a simple button that calls a method when hit
        * label [string]: text shown on the button
        * callback [function]: function with four params (dataset, item, value, parent)
            - dataset [DataSet]: instance of the parent dataset
            - item [DataItem]: instance of ButtonItem (i.e. self)
            - value [unspecified]: value of ButtonItem (default ButtonItem
              value or last value returned by the callback)
            - parent [QObject]: button's parent widget
        * icon [QIcon or string]: icon show on the button (optional)
          (string: icon filename as in guidata/guiqwt image search paths)
        * default [unspecified]: default value passed to the callback (optional)
        * check [bool]: if False, value is not checked (optional, default=True)

    The value of this item is unspecified but is passed to the callback along
    with the whole dataset. The value is assigned the callback`s return value.
    """

    def __init__(self, label, callback, icon=None, default=None, check=True):
        super().__init__("", default=default, check=check)
        self.widget = widgets.PushButton(label)
        if default is not None:
            self.widget.set_value(default)


class DataSetMeta(type):
    """
    DataSet metaclass

    Create class attribute `_items`: list of the DataSet class attributes,
    created in the same order as these attributes were written
    """
    def __new__(mcs, name, bases, dct):
        filtered = [b for b in bases if getattr(b, "__metaclass__", None) is DataSetMeta]
        items = {item._name: item for b in filtered for item in b._items}
        for attrname, value in list(dct.items()):
            if isinstance(value, DataItem):
                items[attrname] = value
        dct["_items"] = items
        return type.__new__(mcs, name, bases, dct)


class DataSet(object, metaclass=DataSetMeta):

    def __init__(self, title=None, comment=None, icon=""):
        # self.widget = custom_widgets.SettingsWidget()
        self.dialog = widgets.BaseDialog()
        self.dialog.set_modality("application")
        self.dialog.setMinimumWidth(400)
        self.dialog.title = title
        self.dialog.set_icon(icon)
        self.dialog.set_layout("grid")
        self.dialog.box.setSpacing(20)
        self.dialog.box.set_margin(20)
        for i, (k, item) in enumerate(self._items.items()):
            self.dialog.box[i, item.label_col] = widgets.Label(item.label)
            self.dialog.box[i, item.label_col + 1:item.label_col + 3] = item.widget
            item.widget.id = k
            for active in item.active_on:
                item.widget.setEnabled(self._items[active].widget.get_value())
                self._items[active].widget.value_changed.connect(item.widget.setEnabled)
            for active in item.not_active_on:
                item.widget.setDisabled(self._items[active].widget.get_value())
                self._items[active].widget.value_changed.connect(item.widget.setDisabled)
        if comment:
            label = widgets.Label(comment)
            label.setWordWrap(True)
            self.dialog.box[i + 1, 0:3] = label
        self.dialog.add_buttonbox()
        # bbox = widgets.DialogButtonBox(widgets.DialogButtonBox.Ok |
        #                                widgets.DialogButtonBox.Cancel)
        # self.dialog.box += bbox
        # bbox.accepted.connect(self.dialog.accept)
        # bbox.rejected.connect(self.dialog.reject)

    def edit(self):
        return self.dialog.show_blocking()

    def to_dict(self):
        return {item.id: item.get_value()
                for item in self.dialog.layout()
                if item.id}

    def from_dict(self, dct):
        for item in self.dialog.layout():
            if item.id in dct and dct[item.id] is not None:
                item.set_value(dct[item.id])


if __name__ == "__main__":
    app = widgets.app()

    class Test(DataSet):
        boolitem = BoolItem(label="My first one")
        stringitem = ChoiceItem(label="A", choices=["A", "B"]).set_not_active("boolitem")
        floatitem = FloatItem(label="My first one").set_active("boolitem")

    dlg = Test(icon="mdi.timer")
    # dlg.widget.value_changed.connect(print)
    if dlg.edit():
        from pprint import pprint
        pprint(dlg.to_dict())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import gui


QtWidgets.QStylePainter.__bases__ = (gui.Painter,)


class StylePainter(QtWidgets.QStylePainter):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtGui

from prettyqt import core, gui
from prettyqt.utils import bidict


PEN_TYPES = bidict(none=QtCore.Qt.NoPen)

COMP_MODES = bidict(source_over=QtGui.QPainter.CompositionMode_SourceOver,
                    destination_over=QtGui.QPainter.CompositionMode_DestinationOver,
                    clear=QtGui.QPainter.CompositionMode_Clear,
                    source=QtGui.QPainter.CompositionMode_Source,
                    destination=QtGui.QPainter.CompositionMode_Destination,
                    source_in=QtGui.QPainter.CompositionMode_SourceIn,
                    destination_in=QtGui.QPainter.CompositionMode_DestinationIn,
                    source_out=QtGui.QPainter.CompositionMode_SourceOut,
                    destination_out=QtGui.QPainter.CompositionMode_DestinationOut,
                    source_atop=QtGui.QPainter.CompositionMode_SourceAtop,
                    destination_atop=QtGui.QPainter.CompositionMode_DestinationAtop)

PATTERNS = bidict(solid=QtCore.Qt.SolidPattern,
                  none=QtCore.Qt.NoBrush,
                  cross=QtCore.Qt.CrossPattern,
                  linear_gradient=QtCore.Qt.LinearGradientPattern,
                  radial_gradient=QtCore.Qt.RadialGradientPattern)


class Painter(QtGui.QPainter):

    def draw_image(self, point, frame_buffer):
        self.set_composition_mode("source_atop")
        self.drawImage(point, frame_buffer)
        self.end()

    def use_antialiasing(self):
        self.setRenderHint(self.Antialiasing, True)

    def fill_rect(self, rect, color, pattern="solid"):
        if isinstance(rect, tuple):
            rect = core.Rect(*rect)
        if isinstance(color, str):
            if color not in gui.Color.colorNames():
                raise ValueError("Invalid value for color.")
            color = gui.Color(color)
        if pattern != "solid":
            color = gui.Brush(color, PATTERNS[pattern])
        self.fillRect(rect, color)

    def set_pen(self, pen_type: str):
        """set pen type to use

        Allowed values are "none",

        Args:
            pen_type: pen type to use

        Raises:
            ValueError: pen type does not exist
        """
        if pen_type not in PEN_TYPES:
            raise ValueError("Invalid value for pen_type.")
        self.setPen(PEN_TYPES[pen_type])

    def get_pen(self) -> str:
        """returns current pen type

        Possible values: "none",

        Returns:
            pen type
        """
        return PEN_TYPES.inv[self.pen()]

    def set_color(self, color):
        color = QtGui.QColor(color)
        self.setPen(color)

    def set_composition_mode(self, mode: str):
        """set the current composition mode

        Possible values: "source_over", "destination_over", "clear", "source",
                         "destination", "source_in", "destination_in", "source_out",
                         "destination_out", "source_atop", "destination_atop",

        Raises:
            ValueError: composition mode does not exist
        """
        if mode not in COMP_MODES:
            raise ValueError("Invalid value for composition mode.")
        self.setCompositionMode(COMP_MODES[mode])

    def get_composition_mode(self) -> str:
        """get the current composition mode

        Possible values: "source_over", "destination_over", "clear", "source",
                         "destination", "source_in", "destination_in", "source_out",
                         "destination_out", "source_atop", "destination_atop",

        Returns:
            composition mode
        """
        return COMP_MODES.inv[self.compositionMode()]

    def draw_text(self, x, y, text):
        self.drawText(x, y, text)



# -*- coding: utf-8 -*-

"""gui module

contains QtGui-based classes
"""

from .guiapplication import GuiApplication
from .validator import Validator
from .regexpvalidator import RegExpValidator
from .intvalidator import IntValidator
from .doublevalidator import DoubleValidator
from .brush import Brush
from .color import Color
from .font import Font
from .fontmetrics import FontMetrics
from .icon import Icon
from .pen import Pen
from .picture import Picture
from .pixmap import Pixmap
from .painter import Painter
from .palette import Palette
from .cursor import Cursor
from .polygonf import PolygonF
from .standarditem import StandardItem
from .standarditemmodel import StandardItemModel
from .textcharformat import TextCharFormat
from .syntaxhighlighter import SyntaxHighlighter
from .pdfwriter import PdfWriter
from .keysequence import KeySequence


__all__ = ["GuiApplication",
           "Validator",
           "RegExpValidator",
           "IntValidator",
           "DoubleValidator",
           "Brush",
           "Color",
           "Font",
           "FontMetrics",
           "Icon",
           "Pen",
           "Picture",
           "Pixmap",
           "Painter",
           "Palette",
           "Cursor",
           "PolygonF",
           "StandardItem",
           "StandardItemModel",
           "TextCharFormat",
           "SyntaxHighlighter",
           "PdfWriter",
           "KeySequence"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from contextlib import contextmanager

from qtpy import QtCore


class Object(QtCore.QObject):

    def __getstate__(self):
        return dict(object_name=self.objectName())

    def __setstate__(self, state):
        self.__init__()
        self.id = state["object_name"]

    @contextmanager
    def block_signals(self):
        self.blockSignals(True)
        yield None
        self.blockSignals(False)

    def to_json(self):
        dct = self.__getstate__()
        for k, v in dct.items():
            if isinstance(v, QtCore.QObject):
                dct[k] = v.to_json()
        return dct

    def set_object_name(self, name: str):
        self.setObjectName(name)

    @property
    def id(self) -> str:
        return self.objectName()

    @id.setter
    def id(self, name: str):
        self.setObjectName(name)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtGui

from prettyqt import core, widgets


class PdfWriter(QtGui.QPdfWriter):

    def setup(self, size):
        dpi = widgets.DesktopWidget().logicalDpiX()
        self.setResolution(dpi)
        self.setPageMargins(QtCore.QMarginsF(0, 0, 0, 0))
        self.setPageSizeMM(core.SizeF(size.width(), size.height()) / dpi * 25.4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import find_packages, setup
from sphinx.setup_command import BuildDoc

with open("README.rst") as readme_file:
    readme = readme_file.read()

with open("HISTORY.rst") as history_file:
    history = history_file.read()

requirements = ["qtpy", "Markdown", "docutils", "qtawesome", "bidict"]

setup_requirements = ["pytest-runner", ]

test_requirements = ["pytest", "pytest-xvfb", "pytest-qt"]

setup(
    author="Philipp Temminghoff",
    author_email="phil65@kodi.tv",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    description="Pythonic layer on top of PyQt5 / PySide2",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + "\n\n" + history,
    long_description_content_type="text/x-rst",
    include_package_data=True,
    keywords="prettyqt",
    name="prettyqt",
    packages=find_packages(),
    setup_requires=setup_requirements,
    test_suite="tests",
    tests_require=test_requirements,
    url="https://github.com/phil65/prettyqt",
    version="0.43.0",
    zip_safe=False,
    cmdclass={"build_sphinx": BuildDoc}
)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets

from prettyqt import gui, widgets
from prettyqt.utils import bidict


H_ALIGNMENTS = bidict(left=QtCore.Qt.AlignLeft,
                      right=QtCore.Qt.AlignRight,
                      center=QtCore.Qt.AlignHCenter,
                      justify=QtCore.Qt.AlignJustify)

V_ALIGNMENTS = bidict(top=QtCore.Qt.AlignTop,
                      bottom=QtCore.Qt.AlignBottom,
                      center=QtCore.Qt.AlignVCenter,
                      baseline=QtCore.Qt.AlignBaseline)


QtWidgets.QSplashScreen.__bases__ = (widgets.Widget,)


class SplashScreen(QtWidgets.QSplashScreen):

    def __init__(self, path, width=None):
        pix = gui.Pixmap(str(path))
        if width:
            pix = pix.scaledToWidth(width)
        super().__init__(pix)
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint |
                            QtCore.Qt.FramelessWindowHint)
        self.setEnabled(False)

    def __enter__(self):
        self.show()
        return self

    def __exit__(self, typ, value, traceback):
        self.hide()

    def set_text(self, text, color="black", h_align="center", v_align="bottom"):
        self.showMessage(text,
                         color=gui.Color(color),
                         alignment=H_ALIGNMENTS[h_align] | V_ALIGNMENTS[v_align])



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore

from prettyqt import core


class DateTime(QtCore.QDateTime):

    def __repr__(self):
        return f"DateTime({self.date()}, {self.time()})"

    def __reduce__(self):
        return (self.__class__, (self.date(), self.time()))


if __name__ == "__main__":
    date = core.Date(2000, 11, 11)
    dt = DateTime(date)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets, core


QtWidgets.QScrollBar.__bases__ = (widgets.AbstractSlider,)


class ScrollBar(QtWidgets.QScrollBar):
    value_changed = core.Signal(int)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QWizardPage.__bases__ = (widgets.Widget,)


class WizardPage(QtWidgets.QWizardPage):
    pass



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


QtWidgets.QTreeView.__bases__ = (widgets.AbstractItemView,)


class TreeView(QtWidgets.QTreeView):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.id = self.__class__.__name__

        # visual settings
        self.setAnimated(True)
        self.setRootIsDecorated(False)
        self.setAllColumnsShowFocus(True)
        self.setUniformRowHeights(True)
        self.setAlternatingRowColors(True)
        self.setWordWrap(False)

        # misc
        self.h_header = widgets.HeaderView("horizontal", parent=self)
        self.set_selection_mode("extended")

    @property
    def h_header(self):
        return self.header()

    @h_header.setter
    def h_header(self, header):
        self.setHeader(header)

    def setup_list_style(self):
        self.setSelectionBehavior(self.SelectRows)
        self.h_header.setStretchLastSection(True)

    def adapt_sizes(self):
        model = self.model()
        if model is not None and (model.rowCount() * model.columnCount()) < 1000:
            self.h_header.resizeSections(self.h_header.ResizeToContents)
        else:
            self.h_header.resize_sections("interactive")


if __name__ == "__main__":
    app = QtWidgets.QApplication([])
    dlg = QtWidgets.QMainWindow()
    status_bar = TreeView()
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore, QtWidgets

from prettyqt import widgets

H_ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    center=QtCore.Qt.AlignHCenter)


QtWidgets.QGroupBox.__bases__ = (widgets.Widget,)


class GroupBox(QtWidgets.QGroupBox):
    """GroupBox widget

    A group box provides a frame, a title on top, a keyboard shortcut,
    and displays various other widgets inside itself.
    The keyboard shortcut moves keyboard focus to one of the group box's child widgets.
    """

    def __repr__(self):
        return f"GroupBox({self.title()!r})"

    def __getstate__(self):
        return dict(checkable=self.isCheckable(),
                    checked=self.isChecked(),
                    tooltip=self.toolTip(),
                    layout=self.layout(),
                    flat=self.isFlat(),
                    # alignment=self.alignment(),
                    title=self.title())

    def __setstate__(self, state):
        self.__init__(state["title"])
        self.set_layout(state["layout"])
        self.setCheckable(state["checkable"])
        self.setChecked(state.get("checked", False))
        self.setFlat(state["flat"])
        self.setToolTip(state.get("tooltip", ""))
        # self.setAlignment(state["alignment"])

    def set_alignment(self, alignment):
        self.setAlignment(H_ALIGNMENTS[alignment])

    def set_enabled(self, state):
        for widget in self.layout():
            widget.setEnabled(state)


if __name__ == "__main__":
    app = widgets.app()
    widget = GroupBox()
    ly = widgets.BoxLayout()
    ly += widgets.RadioButton("test")
    widget.set_layout(ly)
    widget.show()
    app.exec_()
